use std::{
    collections::HashMap,
    env,
    time::{self, Duration},
};

use rs48_lib::{game::Game, grid_displayer::GridDisplayer, prelude::*};
use termion::{clear, cursor};
use tokio::{
    io::AsyncWriteExt,
    net::{tcp::OwnedWriteHalf, TcpListener},
    time::sleep,
};

#[derive(Debug, Clone)]
struct Config {
    delay: Duration,
}

async fn game(mut write: OwnedWriteHalf) -> Option<()> {
    'session: loop {
        let rules = GameRules::default();
        let mut game = Game::new(rules);
        let mut controller = SimulatedController::new(50, 20);
        let displayer = GridDisplayer::new(1234);
        '_game: loop {
            if let None = turn(&mut game, &displayer, &mut write, &mut controller).await {
                continue 'session;
            }
        }
    }
}

async fn turn(
    game: &mut Game,
    displayer: &GridDisplayer,
    write: &mut OwnedWriteHalf,
    controller: &mut SimulatedController,
) -> Option<()> {
    let grid = game.get_board();
    let display = displayer.display(&grid);
    let clear = clear::All;
    let goto = cursor::Goto(1, 1);
    let frame = format!("{clear}{goto}{display}");
    write.write_all(frame.as_bytes()).await.ok()?;
    let movement = controller.next_move(&*game).ok()?;
    game.turn(movement).ok()?;
    sleep(Duration::from_millis(500)).await;
    Some(())
}

#[tokio::main]
async fn main() {
    let url = env::var("URL").unwrap_or_else(|_| "localhost:2048".to_string());
    let delay = env::var("DELAY")
        .map(|s| s.parse::<i32>().unwrap())
        .unwrap_or(500);
    let convif = { delay };
    let server = TcpListener::bind(url).await.unwrap();
    loop {
        let (stream, address) = server.accept().await.unwrap();
        println!("[server] connection from '{address}'");
        let (_, write) = stream.into_split();
        tokio::spawn(game(write));
    }
}
